<?php

namespace Drupal\custom_field\Plugin\CustomFieldType;

use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Field\FieldItemListInterface;

/**
 * Plugin implementation of the 'decimal' custom field type.
 *
 * @CustomFieldType(
 *   id = "decimal",
 *   label = @Translation("Decimal"),
 *   description = @Translation(""),
 *   category = @Translation("Number"),
 *   data_types = {
 *     "decimal",
 *   },
 * )
 */
class Decimal extends NumericBase {

  /**
   * {@inheritdoc}
   */
  public static function defaultWidgetSettings(): array {
    return [
      'settings' => [
        'scale' => '',
      ]
    ] + parent::defaultWidgetSettings();
  }

  /**
   * {@inheritdoc}
   */
  public function widget(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    // Get the base form element properties.
    $element = parent::widget($items, $delta, $element, $form, $form_state);
    $settings = $this->getWidgetSetting('settings');
    $scale = $settings['scale'] ?: $this->configuration['scale'];

    $element['#step'] = pow(0.1, $scale);

    return $element;
  }

  /**
   * {@inheritdoc}
   */
  public function widgetSettingsForm(array $form, FormStateInterface $form_state): array {

    $element = parent::widgetSettingsForm($form, $form_state);
    $settings = $this->widget_settings['settings'] + self::defaultWidgetSettings()['settings'];

    $element['settings']['scale'] = [
      '#type' => 'number',
      '#title' => t('Scale', [], ['context' => 'decimal places']),
      '#min' => 0,
      '#max' => $this->configuration['scale'],
      '#default_value' => $settings['scale'],
      '#description' => t('The number of digits to the right of the decimal.'),
      '#suffix' => $element['settings']['max']['#suffix'],
    ];

    $element['settings']['min']['#step'] = pow(0.1, $this->configuration['scale']);
    $element['settings']['max']['#step'] = pow(0.1, $this->configuration['scale']);

    unset($element['settings']['max']['#suffix']);

    return $element;
  }

}
